import numpy as np

# MQTT params
BROKER_HOST = '127.0.0.1'
URL_INFLUXDB = 'http://100.90.57.1:8086'
BROKER_PORT = 1883
PRECISION = np.float32


PACKAGE_CHANNEL = 0
CH_NAMES = ['C1', 'C2', 'C3', 'C4', 'C5', 'C6', 'C7', 'C8', 'AX', 'AY', 'AZ', 'TIME', 'PKG', 'MARKERS']

# Real stream
EMG_CHANNELS = [1, 2, 3, 4, 5, 6, 7,8 ]  # 
ACC_CHANNELS = [9, 10, 11]
TIME_CHANNEL = 22
MARKER_CHANNEL = 23
NUM_CHANNELS  = len(EMG_CHANNELS + ACC_CHANNELS + [TIME_CHANNEL]+ [PACKAGE_CHANNEL]  + [MARKER_CHANNEL])
ALL_CHANNELS = EMG_CHANNELS + ACC_CHANNELS + [TIME_CHANNEL] + [PACKAGE_CHANNEL ] + [MARKER_CHANNEL]

# synthetic streamn
SYN_EMG_CHANNELS = [1, 2, 3, 4, 5, 6, 7, 8]  #
SYN_ACC_CHANNELS = [17, 18, 19]
SYN_TIME_CHANNEL = 30
SYN_MARKER_CHANNEL = 31
SYN_NUM_CHANNELS = len(SYN_EMG_CHANNELS + SYN_ACC_CHANNELS + [SYN_TIME_CHANNEL] + [PACKAGE_CHANNEL] + [SYN_MARKER_CHANNEL])
SYN_ALL_CHANNELS = SYN_EMG_CHANNELS + SYN_ACC_CHANNELS + [SYN_TIME_CHANNEL] + [PACKAGE_CHANNEL ] + [SYN_MARKER_CHANNEL]

SAMPLE_RATE = 250

# INFLUX
TOPIC = 'data'
TOKEN = ''
INFLUXDB_TOKEN= "holas"
INFLUXDB_ORG = "Ciervo"
INFLUXDB_BUCKET = "Ciervo"